package com.fastbee.data.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.fastbee.common.annotation.Log;
import com.fastbee.common.core.controller.BaseController;
import com.fastbee.common.core.domain.AjaxResult;
import com.fastbee.common.enums.BusinessType;
import com.fastbee.iot.domain.DeviceReportInfo;
import com.fastbee.iot.service.IDeviceReportInfoService;
import com.fastbee.common.utils.poi.ExcelUtil;
import com.fastbee.common.core.page.TableDataInfo;

/**
 * 设备上电审核前上报的基础信息Controller
 *
 * @author kerwincui
 * &#064;date  2024-12-05
 */
@RestController
@RequestMapping("/iot/device/report/info")
@Api(tags = "设备上电审核前上报的基础信息")
public class DeviceReportInfoController extends BaseController
{
    @Autowired
    private IDeviceReportInfoService deviceReportInfoService;

/**
 * 查询设备上电审核前上报的基础信息列表
 */
//    @PreAuthorize("@ss.hasPermi('iot:info:list')")
    @GetMapping("/list")
    @ApiOperation("查询设备上电审核前上报的基础信息列表")
    public TableDataInfo list(DeviceReportInfo deviceReportInfo)
    {
        startPage();
        List<DeviceReportInfo> list = deviceReportInfoService.selectDeviceReportInfoList(deviceReportInfo);
        return getDataTable(list);
    }

    /**
     * 导出设备上电审核前上报的基础信息列表
     */
    @ApiOperation("导出设备上电审核前上报的基础信息列表")
    @PreAuthorize("@ss.hasPermi('iot:info:export')")
    @PostMapping("/export")
    public void export(HttpServletResponse response, DeviceReportInfo deviceReportInfo)
    {
        List<DeviceReportInfo> list = deviceReportInfoService.selectDeviceReportInfoList(deviceReportInfo);
        ExcelUtil<DeviceReportInfo> util = new ExcelUtil<DeviceReportInfo>(DeviceReportInfo.class);
        util.exportExcel(response, list, "设备上电审核前上报的基础信息数据");
    }

    /**
     * 获取设备上电审核前上报的基础信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('iot:info:query')")
    @GetMapping(value = "/{id}")
    @ApiOperation("获取设备上电审核前上报的基础信息详细信息")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(deviceReportInfoService.selectDeviceReportInfoById(id));
    }

    /**
     * 新增设备上电审核前上报的基础信息
     */
    @PreAuthorize("@ss.hasPermi('iot:info:add')")
    @PostMapping
    @ApiOperation("新增设备上电审核前上报的基础信息")
    public AjaxResult add(@RequestBody DeviceReportInfo deviceReportInfo)
    {
        return toAjax(deviceReportInfoService.insertDeviceReportInfo(deviceReportInfo));
    }

    /**
     * 修改设备上电审核前上报的基础信息
     */
    @PreAuthorize("@ss.hasPermi('iot:info:edit')")
    @PutMapping
    @ApiOperation("修改设备上电审核前上报的基础信息")
    public AjaxResult edit(@RequestBody DeviceReportInfo deviceReportInfo)
    {
        return toAjax(deviceReportInfoService.updateDeviceReportInfo(deviceReportInfo));
    }

    /**
     * 删除设备上电审核前上报的基础信息
     */
    @PreAuthorize("@ss.hasPermi('iot:info:remove')")
    @DeleteMapping("/{ids}")
    @ApiOperation("删除设备上电审核前上报的基础信息")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(deviceReportInfoService.deleteDeviceReportInfoByIds(ids));
    }

    /**
     * 批量更新状态为已打印
     */
    @PreAuthorize("@ss.hasPermi('iot:info:edit')")
    @PutMapping("/updateStatus")
    @ApiOperation("批量更新状态")
    public AjaxResult updateStatus(@RequestBody List<DeviceReportInfo> deviceReportInfos){
        return toAjax(deviceReportInfoService.updateDeviceReportStatus(deviceReportInfos));
    }
}
